@@ -30,6 +30,7 @@
 #include "gdkx.h"
 #include "gdk.h"		/* For gdk_error_trap_push()/pop() */
 #include "gdkdisplay-x11.h"
+#include "gdkalias.h"
 
 #include <string.h>
 
@@ -382,10 +383,8 @@ gint
 _gdk_input_common_init (GdkDisplay *display,
 			gint        include_core)
 {
-  char **extensions;
   XDeviceInfo   *devices;
-  int num_devices;
-  int num_extensions, loop;
+  int num_devices, loop;
   int ignore, event_base;
   GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
 
@@ -573,6 +572,8 @@ _gdk_input_common_other_event (GdkEvent         *event,
       gdk_input_translate_coordinates (gdkdev,input_window, xdbe->axis_data,
 				       event->button.axes, 
 				       &event->button.x,&event->button.y);
+      event->button.x_root = event->button.x + input_window->root_x;
+      event->button.y_root = event->button.y + input_window->root_y;
       event->button.state = gdk_input_translate_state(xdbe->state,xdbe->device_state);
       event->button.button = xdbe->button;
 
@@ -669,6 +670,8 @@ _gdk_input_common_other_event (GdkEvent         *event,
       gdk_input_translate_coordinates(gdkdev,input_window,xdme->axis_data,
 				      event->motion.axes,
 				      &event->motion.x,&event->motion.y);
+      event->motion.x_root = event->motion.x + input_window->root_x;
+      event->motion.y_root = event->motion.y + input_window->root_y;
 
       event->motion.type = GDK_MOTION_NOTIFY;
       event->motion.window = input_window->window;
@@ -830,3 +833,6 @@ gdk_device_get_state (GdkDevice       *device,
       XFreeDeviceState (state);
     }
 }
+
+#define __GDK_INPUT_X11_C__
+#include "gdkaliasdef.c"
