@@ -763,6 +763,7 @@ gtk_file_chooser_button_set_property (GObject      *object,
     case GTK_FILE_CHOOSER_PROP_USE_PREVIEW_LABEL:
     case GTK_FILE_CHOOSER_PROP_EXTRA_WIDGET:
     case GTK_FILE_CHOOSER_PROP_SHOW_HIDDEN:
+    case GTK_FILE_CHOOSER_PROP_DO_OVERWRITE_CONFIRMATION:
       g_object_set_property (G_OBJECT (priv->dialog), pspec->name, value);
       break;
 
@@ -809,6 +810,7 @@ gtk_file_chooser_button_get_property (GObject    *object,
     case GTK_FILE_CHOOSER_PROP_EXTRA_WIDGET:
     case GTK_FILE_CHOOSER_PROP_SELECT_MULTIPLE:
     case GTK_FILE_CHOOSER_PROP_SHOW_HIDDEN:
+    case GTK_FILE_CHOOSER_PROP_DO_OVERWRITE_CONFIRMATION:
       g_object_get_property (G_OBJECT (priv->dialog), pspec->name, value);
       break;
 
@@ -1107,6 +1109,9 @@ change_icon_theme (GtkFileChooserButton *button)
       gtk_list_store_set (GTK_LIST_STORE (priv->model), &iter,
 			  ICON_COLUMN, pixbuf,
 			  -1);
+
+      if (pixbuf)
+	g_object_unref (pixbuf);
     }
   while (gtk_tree_model_iter_next (priv->model, &iter));
 
@@ -1309,7 +1314,8 @@ model_add_special (GtkFileChooserButton *button)
 			  DATA_COLUMN, path,
 			  -1);
 
-      g_object_unref (pixbuf);
+      if (pixbuf)
+	g_object_unref (pixbuf);
       button->priv->n_special++;
 
 #ifndef G_OS_WIN32
@@ -1337,7 +1343,8 @@ model_add_special (GtkFileChooserButton *button)
 			  DATA_COLUMN, path,
 			  -1);
 
-      g_object_unref (pixbuf);
+      if (pixbuf)
+	g_object_unref (pixbuf);
       button->priv->n_special++;
     }
 }
@@ -1377,7 +1384,8 @@ model_add_volumes (GtkFileChooserButton *button,
 			  DATA_COLUMN, volumes->data,
 			  -1);
 
-      g_object_unref (pixbuf);
+      if (pixbuf)
+	g_object_unref (pixbuf);
       g_free (display_name);
 
       button->priv->n_volumes++;
