@@ -28,7 +28,6 @@
 
 #include <config.h>
 
-#include "gtkalias.h"
 #include "gtkfontbutton.h"
 
 #include "gtksignal.h"
@@ -40,7 +39,9 @@
 #include "gtkfontsel.h"
 #include "gtkimage.h"
 #include "gtkmarshalers.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -176,8 +177,8 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
                                                         P_("Title"),
                                                         P_("The title of the font selection dialog"),
                                                         _("Pick a Font"),
-                                                        (G_PARAM_READABLE |
-                                                         G_PARAM_WRITABLE)));
+                                                        (GTK_PARAM_READABLE |
+                                                         GTK_PARAM_WRITABLE)));
 
   /**
    * GtkFontButton:font-name:
@@ -188,12 +189,12 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_FONT_NAME,
-                                   g_param_spec_string ("font_name",
+                                   g_param_spec_string ("font-name",
                                                         P_("Font name"),
                                                         P_("The name of the selected font"),
                                                         P_("Sans 12"),
-                                                        (G_PARAM_READABLE |
-                                                         G_PARAM_WRITABLE)));
+                                                        (GTK_PARAM_READABLE |
+                                                         GTK_PARAM_WRITABLE)));
 
   /**
    * GtkFontButton:use-font:
@@ -204,12 +205,11 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_USE_FONT,
-                                   g_param_spec_boolean ("use_font",
+                                   g_param_spec_boolean ("use-font",
                                                          P_("Use font in label"),
                                                          P_("Whether the label is drawn in the selected font"),
                                                          FALSE,
-                                                         (G_PARAM_READABLE |
-                                                          G_PARAM_WRITABLE)));
+                                                         GTK_PARAM_READWRITE));
 
   /**
    * GtkFontButton:use-size:
@@ -220,12 +220,11 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_USE_SIZE,
-                                   g_param_spec_boolean ("use_size",
+                                   g_param_spec_boolean ("use-size",
                                                          P_("Use size in label"),
                                                          P_("Whether the label is drawn with the selected font size"),
                                                          FALSE,
-                                                         (G_PARAM_READABLE |
-                                                          G_PARAM_WRITABLE)));
+                                                         GTK_PARAM_READWRITE));
 
   /**
    * GtkFontButton:show-style:
@@ -237,12 +236,11 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_STYLE,
-                                   g_param_spec_boolean ("show_style",
+                                   g_param_spec_boolean ("show-style",
                                                          P_("Show style"),
                                                          P_("Whether the selected font style is shown in the label"),
                                                          TRUE,
-                                                         (G_PARAM_READABLE |
-                                                          G_PARAM_WRITABLE)));
+                                                         GTK_PARAM_READWRITE));
   /**
    * GtkFontButton:show-size:
    * 
@@ -253,12 +251,11 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_SIZE,
-                                   g_param_spec_boolean ("show_size",
+                                   g_param_spec_boolean ("show-size",
                                                          P_("Show size"),
                                                          P_("Whether selected font size is shown in the label"),
                                                          TRUE,
-                                                         (G_PARAM_READABLE |
-                                                          G_PARAM_WRITABLE)));
+                                                         GTK_PARAM_READWRITE));
 
   /**
    * GtkFontButton::font-set:
@@ -269,7 +266,7 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    *
    * Since: 2.4
    */
-  font_button_signals[FONT_SET] = g_signal_new ("font_set",
+  font_button_signals[FONT_SET] = g_signal_new ("font-set",
                                                 G_TYPE_FROM_CLASS (gobject_class),
                                                 G_SIGNAL_RUN_FIRST,
                                                 G_STRUCT_OFFSET (GtkFontButtonClass, font_set),
@@ -508,7 +505,7 @@ gtk_font_button_set_use_font (GtkFontButton *font_button,
       else
 	gtk_widget_set_style (font_button->priv->font_label, NULL);
  
-     g_object_notify (G_OBJECT (font_button), "use_font");
+     g_object_notify (G_OBJECT (font_button), "use-font");
     }
 } 
 
@@ -554,7 +551,7 @@ gtk_font_button_set_use_size (GtkFontButton *font_button,
       if (font_button->priv->use_font)
         gtk_font_button_label_use_font (font_button);
 
-      g_object_notify (G_OBJECT (font_button), "use_size");
+      g_object_notify (G_OBJECT (font_button), "use-size");
     }
 } 
 
@@ -598,7 +595,7 @@ gtk_font_button_set_show_style (GtkFontButton *font_button,
       
       gtk_font_button_update_font_info (font_button);
   
-      g_object_notify (G_OBJECT (font_button), "show_style");
+      g_object_notify (G_OBJECT (font_button), "show-style");
     }
 } 
 
@@ -648,7 +645,7 @@ gtk_font_button_set_show_size (GtkFontButton *font_button,
       
       gtk_font_button_update_font_info (font_button);
 
-      g_object_notify (G_OBJECT (font_button), "show_size");
+      g_object_notify (G_OBJECT (font_button), "show-size");
     }
 } 
 
@@ -708,7 +705,7 @@ gtk_font_button_set_font_name (GtkFontButton *font_button,
   else
     result = FALSE;
 
-  g_object_notify (G_OBJECT (font_button), "font_name");
+  g_object_notify (G_OBJECT (font_button), "font-name");
 
   return result;
 }
@@ -769,7 +766,7 @@ dialog_ok_clicked (GtkWidget *widget,
   /* Set label font */
   gtk_font_button_update_font_info (font_button);
 
-  g_object_notify (G_OBJECT (font_button), "font_name");
+  g_object_notify (G_OBJECT (font_button), "font-name");
   
   /* Emit font_set signal */
   g_signal_emit (font_button, font_button_signals[FONT_SET], 0);
@@ -872,7 +869,7 @@ gtk_font_button_update_font_info (GtkFontButton *font_button)
 #endif
 
   style = NULL;
-  if (font_button->priv->show_style) 
+  if (font_button->priv->show_style && family) 
     {
       PangoFontFamily **families;
       PangoFontFace **faces;
@@ -936,10 +933,5 @@ gtk_font_button_update_font_info (GtkFontButton *font_button)
   pango_font_description_free (desc);
 } 
 
-
-
-
-
-
-
-
+#define __GTK_FONT_BUTTON_C__
+#include "gtkaliasdef.c"
