@@ -366,13 +366,14 @@ gtk_target_list_add_image_targets (GtkTargetList *list,
   for (f = formats; f; f = f->next)
     {
       GdkPixbufFormat *fmt = f->data;
-      gchar *name;
-
+      gchar *name; 
+ 
       name = gdk_pixbuf_format_get_name (fmt);
       if (strcmp (name, "png") == 0)
 	{
 	  formats = g_slist_delete_link (formats, f);
 	  formats = g_slist_prepend (formats, fmt);
+
 	  g_free (name);
 
 	  break;
@@ -1365,17 +1366,20 @@ gtk_selection_data_set_pixbuf (GtkSelectionData *selection_data,
 	  if (selection_data->target == atom)
 	    {
 	      str = NULL;
-	      type = gdk_pixbuf_format_get_name (fmt),
+	      type = gdk_pixbuf_format_get_name (fmt);
 	      result = gdk_pixbuf_save_to_buffer (pixbuf, &str, &len,
-						  type, NULL, NULL);
-	      if (result) 
+						  type, NULL,
+                                                  ((strcmp (type, "png") == 0) ?
+                                                   "compression" : NULL), "2",
+                                                  NULL);
+	      if (result)
 		gtk_selection_data_set (selection_data,
 					atom, 8, (guchar *)str, len);
 	      g_free (type);
 	      g_free (str);
 	      g_strfreev (mimes);
 	      g_slist_free (formats);
-	      
+
 	      return result;
 	    }
 	}
@@ -1827,6 +1831,8 @@ _gtk_selection_request (GtkWidget *widget,
 	      info->conversions[i].property = gdk_x11_xatom_to_atom_for_display (display,
 										 ((glong *)mult_atoms)[2*i + 1]);
 	    }
+
+	  g_free (mult_atoms);
 	}
       else
 #endif
@@ -2572,26 +2578,26 @@ gtk_selection_default_handler (GtkWidget	*widget,
 
 /**
  * gtk_selection_data_copy:
- * @selection_data: a pointer to a #GtkSelectionData structure.
+ * @data: a pointer to a #GtkSelectionData structure.
  * 
  * Makes a copy of a #GtkSelectionData structure and its data.
  * 
  * Return value: a pointer to a copy of @data.
  **/
 GtkSelectionData*
-gtk_selection_data_copy (GtkSelectionData *selection_data)
+gtk_selection_data_copy (GtkSelectionData *data)
 {
   GtkSelectionData *new_data;
   
-  g_return_val_if_fail (selection_data != NULL, NULL);
+  g_return_val_if_fail (data != NULL, NULL);
   
   new_data = g_new (GtkSelectionData, 1);
-  *new_data = *selection_data;
+  *new_data = *data;
 
-  if (selection_data->data)
+  if (data->data)
     {
-      new_data->data = g_malloc (selection_data->length + 1);
-      memcpy (new_data->data, selection_data->data, selection_data->length + 1);
+      new_data->data = g_malloc (data->length + 1);
+      memcpy (new_data->data, data->data, data->length + 1);
     }
   
   return new_data;
