@@ -18,11 +18,12 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
+#include <string.h>
 #include "gtktreeselection.h"
 #include "gtktreeprivate.h"
 #include "gtkrbtree.h"
 #include "gtkmarshalers.h"
+#include "gtkalias.h"
 
 static void gtk_tree_selection_init              (GtkTreeSelection      *selection);
 static void gtk_tree_selection_class_init        (GtkTreeSelectionClass *class);
@@ -619,7 +620,7 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
   GtkTreeIter iter;
 
   guint inserted_id, deleted_id, reordered_id;
-  gboolean stop = FALSE, has_next = TRUE, has_parent = TRUE;
+  gboolean stop = FALSE;
 
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
@@ -664,59 +665,46 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 
   /* find the node internally */
   path = gtk_tree_path_new_first ();
-  gtk_tree_model_get_iter (selection->tree_view->priv->model,
-			   &iter, path);
 
   do
     {
       if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
-	(* func) (selection->tree_view->priv->model, path, &iter, data);
+        {
+          gtk_tree_model_get_iter (selection->tree_view->priv->model,
+                                   &iter, path);
+	  (* func) (selection->tree_view->priv->model, path, &iter, data);
+        }
 
       if (stop)
 	goto out;
 
       if (node->children)
 	{
-	  gboolean has_child;
-	  GtkTreeIter tmp;
-
 	  tree = node->children;
 	  node = tree->root;
+
 	  while (node->left != tree->nil)
 	    node = node->left;
-	  tmp = iter;
-	  has_child = gtk_tree_model_iter_children (selection->tree_view->priv->model, &iter, &tmp);
-	  gtk_tree_path_append_index (path, 0);
 
-	  /* we do the sanity check at the bottom of this function */
-	  if (!has_child)
-	    goto out;
+	  gtk_tree_path_append_index (path, 0);
 	}
       else
 	{
 	  gboolean done = FALSE;
+
 	  do
 	    {
 	      node = _gtk_rbtree_next (tree, node);
 	      if (node != NULL)
 		{
-		  gboolean has_next;
-
-		  has_next = gtk_tree_model_iter_next (selection->tree_view->priv->model, &iter);
 		  done = TRUE;
 		  gtk_tree_path_next (path);
-
-		  /* we do the sanity check at the bottom of this function */
-		  if (!has_next)
-		    goto out;
 		}
 	      else
 		{
-		  gboolean has_parent;
-		  GtkTreeIter tmp_iter = iter;
-
 		  node = tree->parent_node;
 		  tree = tree->parent_tree;
+
 		  if (tree == NULL)
 		    {
 		      /* we've run out of tree */
@@ -725,12 +713,7 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 		      goto out;
 		    }
 
-		  has_parent = gtk_tree_model_iter_parent (selection->tree_view->priv->model, &iter, &tmp_iter);
 		  gtk_tree_path_up (path);
-
-		  /* we do the sanity check at the bottom of this function */
-		  if (!has_parent)
-		    goto out;
 		}
 	    }
 	  while (!done);
@@ -750,10 +733,6 @@ out:
                                reordered_id);
 
   /* check if we have to spew a scary message */
-  if (!has_next)
-    TREE_VIEW_INTERNAL_ASSERT_VOID (has_next);
-  if (!has_parent)
-    TREE_VIEW_INTERNAL_ASSERT_VOID (has_parent);
   if (stop)
     g_warning
       ("The model has been modified from within gtk_tree_selection_selected_foreach.\n"
@@ -1303,8 +1282,8 @@ tree_column_is_sensitive (GtkTreeViewColumn *column,
   list = cells;
   while (list)
     {
-      g_object_get (G_OBJECT (list->data), 
-		    "sensitive", &sensitive, 
+      g_object_get (list->data,
+		    "sensitive", &sensitive,
 		    "visible", &visible,
 		    NULL);
       
@@ -1542,3 +1521,6 @@ gtk_tree_selection_real_select_node (GtkTreeSelection *selection,
 
   return FALSE;
 }
+
+#define __GTK_TREE_SELECTION_C__
+#include "gtkaliasdef.c"
