@@ -25,7 +25,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <assert.h>
 #include "xdgmimemagic.h"
 #include "xdgmimeint.h"
@@ -44,8 +47,6 @@
 #define	TRUE	(!FALSE)
 #endif
 
-extern int errno;
-
 typedef struct XdgMimeMagicMatch XdgMimeMagicMatch;
 typedef struct XdgMimeMagicMatchlet XdgMimeMagicMatchlet;
 
@@ -168,7 +169,7 @@ _xdg_mime_magic_read_to_newline (FILE *magic_file,
 
   while (TRUE)
     {
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
       if (c == EOF)
 	{
 	  *end_of_file = TRUE;
@@ -203,7 +204,7 @@ _xdg_mime_magic_read_a_number (FILE *magic_file,
 
   while (TRUE)
     {
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
 
       if (c == EOF)
 	{
@@ -247,7 +248,7 @@ _xdg_mime_magic_parse_header (FILE *magic_file, XdgMimeMagicMatch *match)
   assert (magic_file != NULL);
   assert (match != NULL);
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     return XDG_MIME_MAGIC_EOF;
   if (c != '[')
@@ -259,13 +260,13 @@ _xdg_mime_magic_parse_header (FILE *magic_file, XdgMimeMagicMatch *match)
   if (match->priority == -1)
     return XDG_MIME_MAGIC_ERROR;
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     return XDG_MIME_MAGIC_EOF;
   if (c != ':')
     return XDG_MIME_MAGIC_ERROR;
 
-  buffer = _xdg_mime_magic_read_to_newline (magic_file, &end_of_file);
+  buffer = (char *)_xdg_mime_magic_read_to_newline (magic_file, &end_of_file);
   if (end_of_file)
     return XDG_MIME_MAGIC_EOF;
 
@@ -292,7 +293,7 @@ _xdg_mime_magic_parse_error (FILE *magic_file)
 
   while (1)
     {
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
       if (c == EOF)
 	return XDG_MIME_MAGIC_EOF;
       if (c == '\n')
@@ -317,7 +318,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
   assert (magic_file != NULL);
 
   /* Sniff the buffer to make sure it's a valid line */
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     return XDG_MIME_MAGIC_EOF;
   else if (c == '[')
@@ -338,7 +339,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	return XDG_MIME_MAGIC_EOF;
       if (indent == -1)
 	return XDG_MIME_MAGIC_ERROR;
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
       if (c == EOF)
 	return XDG_MIME_MAGIC_EOF;
     }
@@ -359,7 +360,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
       _xdg_mime_magic_matchlet_free (matchlet);
       return XDG_MIME_MAGIC_ERROR;
     }
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     {
       _xdg_mime_magic_matchlet_free (matchlet);
@@ -373,7 +374,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 
   /* Next two bytes determine how long the value is */
   matchlet->value_length = 0;
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     {
       _xdg_mime_magic_matchlet_free (matchlet);
@@ -382,7 +383,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
   matchlet->value_length = c & 0xFF;
   matchlet->value_length = matchlet->value_length << 8;
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     {
       _xdg_mime_magic_matchlet_free (matchlet);
@@ -408,7 +409,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	return XDG_MIME_MAGIC_ERROR;
     }
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == '&')
     {
       matchlet->mask = malloc (matchlet->value_length);
@@ -427,7 +428,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	  else
 	    return XDG_MIME_MAGIC_ERROR;
 	}
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
     }
 
   if (c == '~')
@@ -446,7 +447,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	  _xdg_mime_magic_matchlet_free (matchlet);
 	  return XDG_MIME_MAGIC_ERROR;
 	}
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
     }
 
   if (c == '+')
@@ -462,7 +463,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	  _xdg_mime_magic_matchlet_free (matchlet);
 	  return XDG_MIME_MAGIC_ERROR;
 	}
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
     }
 
 
@@ -517,8 +518,7 @@ _xdg_mime_magic_matchlet_compare_to_data (XdgMimeMagicMatchlet *matchlet,
 					  size_t                len)
 {
   int i, j;
-
-  for (i = matchlet->offset; i <= matchlet->offset + matchlet->range_length; i++)
+  for (i = matchlet->offset; i < matchlet->offset + matchlet->range_length; i++)
     {
       int valid_matchlet = TRUE;
 
@@ -636,8 +636,10 @@ _xdg_mime_magic_new (void)
 void
 _xdg_mime_magic_free (XdgMimeMagic *mime_magic)
 {
-  if (mime_magic)
+  if (mime_magic) {
+    _xdg_mime_magic_match_free (mime_magic->match_list);
     free (mime_magic);
+  }
 }
 
 int
@@ -652,16 +654,20 @@ _xdg_mime_magic_lookup_data (XdgMimeMagic *mime_magic,
 			     size_t        len)
 {
   XdgMimeMagicMatch *match;
+  const char *mime_type;
 
+  mime_type = NULL;
   for (match = mime_magic->match_list; match; match = match->next)
     {
       if (_xdg_mime_magic_match_compare_to_data (match, data, len))
 	{
-	  return match->mime_type;
+	  if ((mime_type == NULL) || (xdg_mime_mime_type_subclass (match->mime_type, mime_type))) {
+	    mime_type = match->mime_type;
+	  }
 	}
     }
 
-  return NULL;
+  return mime_type;
 }
 
 static void
@@ -692,13 +698,13 @@ _xdg_mime_magic_matchlet_mirror (XdgMimeMagicMatchlet *matchlets)
 {
   XdgMimeMagicMatchlet *new_list;
   XdgMimeMagicMatchlet *tmp;
-  
-  if ((matchlets == NULL) || (matchlets->next == NULL)) 
+
+  if ((matchlets == NULL) || (matchlets->next == NULL))
     return matchlets;
 
   new_list = NULL;
   tmp = matchlets;
-  while (tmp != NULL) 
+  while (tmp != NULL)
     {
       XdgMimeMagicMatchlet *matchlet;
 
@@ -709,7 +715,7 @@ _xdg_mime_magic_matchlet_mirror (XdgMimeMagicMatchlet *matchlets)
     }
 
   return new_list;
-  
+
 }
 
 static void
@@ -733,7 +739,8 @@ _xdg_mime_magic_read_magic_file (XdgMimeMagic *mime_magic,
 	  break;
 	case XDG_MIME_MAGIC_MAGIC:
 	  state = _xdg_mime_magic_parse_magic_line (magic_file, match);
-	  if (state == XDG_MIME_MAGIC_SECTION) 
+	  if (state == XDG_MIME_MAGIC_SECTION ||
+	      (state == XDG_MIME_MAGIC_EOF && match->mime_type))
 	    {
 	      match->matchlet = _xdg_mime_magic_matchlet_mirror (match->matchlet);
 	      _xdg_mime_magic_insert_match (mime_magic, match);
@@ -765,9 +772,11 @@ _xdg_mime_magic_read_from_file (XdgMimeMagic *mime_magic,
   if (magic_file == NULL)
     return;
 
-  fread (header, 1, 12, magic_file);
+  if (fread (header, 1, 12, magic_file) == 12)
+    {
+      if (memcmp ("MIME-Magic\0\n", header, 12) == 0)
+        _xdg_mime_magic_read_magic_file (mime_magic, magic_file);
+    }
 
-  if (memcmp ("MIME-Magic\0\n", header, 12) == 0)
-    _xdg_mime_magic_read_magic_file (mime_magic, magic_file);
   fclose (magic_file);
 }
